@@ -1,11 +1,12 @@
 package Lemonldap::NG::Common::Conf::RESTServer;
 
 use strict;
+use JSON 'from_json';
 use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
@@ -100,18 +101,18 @@ sub getConfByNum {
 #    * openIdIDPList() (old OpenID)
 #    * applicationList()
 #  - root:
-#    root query (/confs/latest for example) is redirected to metadatas()
+#    root query (/confs/latest for example) is redirected to metadata()
 #  - other requests:
 #    they are managed by getKey()
 #  - newRSAKey() returns a new RSA key pair if /confs/newRSAKey is called in a
 #    POST request
-#  - prx() load a request and return the content (for SAML/OIDC metadatas)
+#  - prx() load a request and return the content (for SAML/OIDC metadata)
 
 # 31 - Complex subnodes
 #      ----------------
 
 ## @method PSGI-JSON-response complexNodesRoot($req, $query, $tpl)
-# Respond to root requests for virtual hosts and SAMLmetadatas
+# Respond to root requests for virtual hosts and SAMLmetadata
 #
 #@param $req Lemonldap::NG::Common::PSGI::Request
 #@param $query Configuration root key
@@ -194,9 +195,11 @@ sub virtualHosts {
                 type  => 'keyText',
             };
 
-            # If rule contains a comment, split it
+            # If rule contains a comment or an AuthLevel, split them
             if ( $query eq 'locationRules' ) {
                 $res->{comment} = '';
+                $res->{level}   = '';
+                $res->{level}   = $1 if ( $r =~ s/\(\?#AuthnLevel=(-?\d+)\)// );
                 if ( $r =~ s/\(\?#(.*?)\)// ) {
                     $res->{title} = $res->{comment} = $1;
                 }
@@ -211,7 +214,7 @@ sub virtualHosts {
         }
         return $self->sendJSONresponse( $req, $resp );
     }
-    elsif ( $query =~ /^vhost(?:(?:Aliase|Http)s|Maintenance|Port|Type)$/ ) {
+    elsif ( $query =~ qr/^$virtualHostKeys$/o ) {
         $self->logger->debug("Query for $vh/$query key");
 
         # TODO: verify how this is done actually
@@ -281,6 +284,21 @@ sub _samlMetaDataNodes {
         }
         return $self->sendJSONresponse( $req, $resp );
     }
+    elsif ( $query eq "samlSPMetaDataMacros" ) {
+        my $pk =
+          eval { $self->getConfKey( $req, $query )->{$partner} } // {};
+        return $self->sendError( $req, undef, 400 ) if ( $req->error );
+        foreach my $h ( sort keys %$pk ) {
+            push @$resp,
+              {
+                id    => "saml${type}MetaDataNodes/$partner/$query/" . $id++,
+                title => $h,
+                data  => $pk->{$h},
+                type  => 'keyText',
+              };
+        }
+        return $self->sendJSONresponse( $req, $resp );
+    }
 
     # Simple root keys
     elsif ( $query =~ /^saml${type}MetaDataXML$/ ) {
@@ -377,7 +395,7 @@ sub _oidcMetaDataNodes {
 
     # Return all exported attributes if asked
     if ( $query =~
-        /^(?:oidc${type}MetaDataExportedVars|oidcRPMetaDataOptionsExtraClaims)$/
+/^(?:oidc${type}MetaDataExportedVars|oidcRPMetaDataOptionsExtraClaims|oidcRPMetaDataMacros)$/
       )
     {
         my $pk = eval { $self->getConfKey( $req, $query )->{$partner} } // {};
@@ -444,6 +462,84 @@ sub oidcRPMetaDataNodes {
     return $self->_oidcMetaDataNodes( 'RP', $req, @path );
 }
 
+# 314 - CAS
+#       ---
+
+sub _casMetaDataNodes {
+    my ( $self, $type, $req, @path ) = @_;
+    my $refKey =
+      ( $type eq 'App' ? 'casAppMetaDataOptions' : 'casSrvMetaDataOptions' );
+    return $self->complexNodesRoot( $req, $refKey, "cas${type}MetaDataNode" )
+      unless (@path);
+
+    my $partner = shift @path;
+    my $query   = shift @path;
+    unless ($query) {
+        return $self->sendError( $req,
+            "Bad request: cas${type}MetaDataNode query must ask for a key",
+            400 );
+    }
+
+    # setDefault response for new partners
+    return $self->sendError( $req, 'setDefault', 200 )
+      if ( $partner =~ /^new__/ );
+
+    # Reject unknown partners
+    return $self->sendError( $req, "Unknown CAS partner ($partner)", 400 )
+      unless ( defined eval { $self->getConfKey( $req, $refKey )->{$partner}; }
+      );
+
+    my ( $id, $resp ) = ( 1, [] );
+
+    # Return all exported attributes if asked
+    if ( $query =~
+/^(?:cas${type}MetaDataExportedVars|casSrvMetaDataOptionsProxiedServices|casAppMetaDataMacros)$/
+      )
+    {
+        my $pk = eval { $self->getConfKey( $req, $query )->{$partner} } // {};
+        return $self->sendError( $req, undef, 400 ) if ( $req->error );
+        foreach my $h ( sort keys %$pk ) {
+            push @$resp,
+              {
+                id    => "cas${type}MetaDataNodes/$partner/$query/" . $id++,
+                title => $h,
+                data  => $pk->{$h},
+                type  => 'keyText',
+              };
+        }
+        return $self->sendJSONresponse( $req, $resp );
+    }
+
+    # Options
+    if (
+        $query =~ {
+            App => qr/^$casAppMetaDataNodeKeys$/o,
+            Srv => qr/^$casSrvMetaDataNodeKeys$/o
+        }->{$type}
+      )
+    {
+        my $value = eval {
+            $self->getConfKey( $req, "cas${type}MetaDataOptions" )->{$partner}
+              ->{$query};
+        } // undef;
+        return $self->sendJSONresponse( $req, { value => $value } );
+    }
+    else {
+        return $self->sendError( $req,
+            "Bad key for cas${type}MetaDataNode ($query)", 400 );
+    }
+}
+
+sub casSrvMetaDataNodes {
+    my ( $self, $req, @path ) = @_;
+    return $self->_casMetaDataNodes( 'Srv', $req, @path );
+}
+
+sub casAppMetaDataNodes {
+    my ( $self, $req, @path ) = @_;
+    return $self->_casMetaDataNodes( 'App', $req, @path );
+}
+
 # 32 - Other special nodes
 #      -------------------
 
@@ -461,11 +557,23 @@ sub authChoiceModules {
     unless ($key) {
         my @res;
         foreach my $k ( sort keys %$value ) {
+            my $data = [ split /;/, $value->{$k} ];
+            if ( $data->[5] ) {
+                my $over;
+                eval { $over = from_json( $data->[5] ) };
+                if ($@) {
+                    $self->logger->error(
+                        "Bad value in choice over parameters, deleted ($@)");
+                }
+                else {
+                    $data->[5] = [ map { [ $_, $over->{$_} ] } keys %{$over} ];
+                }
+            }
             push @res,
               {
                 id    => "authChoiceModules/$k",
                 title => "$k",
-                data  => [ split /;/, $value->{$k} ],
+                data  => $data,
                 type  => 'authChoice'
               };
         }
@@ -486,7 +594,8 @@ sub authChoiceModules {
 #@return PSGI JSON response
 sub grantSessionRules {
     my ( $self, $req, $key ) = @_;
-    return $self->sendError( 'Subkeys forbidden for grantSessionRules', 400 )
+    return $self->sendError( $req, 'Subkeys forbidden for grantSessionRules',
+        400 )
       if ($key);
     my $value = $self->getConfKey( $req, 'grantSessionRules' );
     my @res;
@@ -520,7 +629,7 @@ sub grantSessionRules {
 # Split openIdIDPList parameter into 2 elements
 sub openIdIDPList {
     my ( $self, $req, $key ) = @_;
-    return $self->sendError( 'Subkeys forbidden for openIdIDPList', 400 )
+    return $self->sendError( $req, 'Subkeys forbidden for openIdIDPList', 400 )
       if ($key);
     my $value = $self->getConfKey( $req, 'openIdIDPList' );
     $value //= '0;';
@@ -559,7 +668,14 @@ sub _scanCatsAndApps {
     my ( $self, $apps, $baseId ) = @_;
     my @res;
 
-    foreach my $cat ( grep { not /^(?:catname|type)$/ } sort keys %$apps ) {
+    foreach my $cat (
+        sort {
+            ( $apps->{$a}->{order} || 0 ) <=> ( $apps->{$b}->{order} || 0 )
+              or $a cmp $b
+        }
+        grep { not /^(?:catname|type|order)$/ } keys %$apps
+      )
+    {
         my $item = { id => "$baseId/$cat" };
         if ( $apps->{$cat}->{type} eq 'category' ) {
             $item->{title} = $apps->{$cat}->{catname};
@@ -569,7 +685,7 @@ sub _scanCatsAndApps {
         }
         else {
             $item->{title} = $apps->{$cat}->{options}->{name};
-            $item->{type} = $apps->{$cat}->{type} = 'menuApp';
+            $item->{type}  = $apps->{$cat}->{type} = 'menuApp';
             foreach my $o (
                 grep { not /^name$/ }
                 keys %{ $apps->{$cat}->{options} }
@@ -588,15 +704,15 @@ sub _scanCatsAndApps {
 # Returns raw value, just transform "over" key
 sub combModules {
     my ( $self, $req, $key ) = @_;
-    return $self->sendError( 'Subkeys forbidden for combModules', 400 )
+    return $self->sendError( $req, 'Subkeys forbidden for combModules', 400 )
       if ($key);
     my $val = $self->getConfKey( $req, 'combModules' ) // {};
     my $res = [];
     foreach my $mod ( keys %$val ) {
         my $tmp;
-        $tmp->{title} = $mod;
-        $tmp->{id}    = "combModules/$mod";
-        $tmp->{type}  = 'cmbModule';
+        $tmp->{title}      = $mod;
+        $tmp->{id}         = "combModules/$mod";
+        $tmp->{type}       = 'cmbModule';
         $tmp->{data}->{$_} = $val->{$mod}->{$_} foreach (qw(type for));
         my $over = $val->{$mod}->{over} // {};
         $tmp->{data}->{over} = [ map { [ $_, $over->{$_} ] } keys %$over ];
@@ -605,18 +721,38 @@ sub combModules {
     return $self->sendJSONresponse( $req, $res );
 }
 
+sub sfExtra {
+    my ( $self, $req, $key ) = @_;
+    return $self->sendError( $req, 'Subkeys forbidden for sfExtra', 400 )
+      if ($key);
+    my $val = $self->getConfKey( $req, 'sfExtra' ) // {};
+    my $res = [];
+    foreach my $mod ( keys %$val ) {
+        my $tmp;
+        $tmp->{title}      = $mod;
+        $tmp->{id}         = "sfExtra/$mod";
+        $tmp->{type}       = 'sfExtra';
+        $tmp->{data}->{$_} = $val->{$mod}->{$_}
+          foreach (qw(type rule logo level label));
+        my $over = $val->{$mod}->{over} // {};
+        $tmp->{data}->{over} = [ map { [ $_, $over->{$_} ] } keys %$over ];
+        push @$res, $tmp;
+    }
+    return $self->sendJSONresponse( $req, $res );
+}
+
 # 33 - Root queries
 #      -----------
 
-## @method PSGI-JSON-response metadatas($req)
-# Respond to `/conf/:cfgNum` requests by sending configuration metadatas
+## @method PSGI-JSON-response metadata($req)
+# Respond to `/conf/:cfgNum` requests by sending configuration metadata
 #
 # NB: if `full=1` is set in the query, configuration is returned directly in
 #     JSON
 #
 #@param $req Lemonldap::NG::Common::PSGI::Request
 #@return PSGI JSON response
-sub metadatas {
+sub metadata {
     my ( $self, $req ) = @_;
     if ( $req->params('full') and $req->params('full') !~ NO ) {
         my $c = $self->getConfKey( $req, 'cfgNum' );
@@ -628,8 +764,7 @@ sub metadatas {
                   . $c );
         }
         else {
-            $self->logger->info(
-                "REST request to get full configuration $c");
+            $self->logger->info("REST request to get full configuration $c");
         }
         return $self->sendJSONresponse(
             $req,
@@ -651,21 +786,23 @@ sub metadatas {
         }
 
         # Find next and previous conf
-        my @a  = $self->confAcc->available;
-        my $id = -1;
+        my @a     = $self->confAcc->available;
+        my $id    = -1;
         my ($ind) = map { $id++; $_ == $res->{cfgNum} ? ($id) : () } @a;
         if ($ind) { $res->{prev} = $a[ $ind - 1 ]; }
-        if ( $ind and $ind < $#a ) { $res->{next} = $a[ $ind + 1 ]; }
+        if ( defined $ind and $ind < $#a ) {
+            $res->{next} = $a[ $ind + 1 ];
+        }
         if ( $self->can('userId') ) {
             $self->userLogger->info( 'User '
                   . $self->userId($req)
-                  . ' ask for configuration metadatas ('
+                  . ' ask for configuration metadata ('
                   . $res->{cfgNum}
                   . ')' );
         }
         else {
             $self->logger->info(
-                "REST request to get configuration metadatas ($res->{cfgNum})");
+                "REST request to get configuration metadata ($res->{cfgNum})");
         }
         return $self->sendJSONresponse( $req, $res );
     }
@@ -684,10 +821,15 @@ sub metadatas {
 sub getKey {
     my ( $self, $req, $key, $subkey ) = @_;
     unless ($key) {
-        return $self->metadatas($req);
+        return $self->metadata($req);
+    }
+    if ( $self->can('userId') ) {
+        $self->userLogger->info(
+            'User ' . $self->userId($req) . " asks for key $key" );
+    }
+    else {
+        $self->logger->info("REST request to get configuration key $key");
     }
-    $self->userLogger->info(
-        'User ' . $self->userId($req) . " asks for key $key" );
     my $value = $self->getConfKey( $req, $key );
     return $self->sendError( $req, undef, 400 ) if ( $req->error );
 
@@ -717,14 +859,13 @@ sub getKey {
     }
     elsif ( $key =~ qr/^$doubleHashKeys$/o ) {
         my @res;
-        if ( defined $value ) {
-            foreach my $host ( sort keys %$value ) {
-                my @tmp;
-                foreach my $k ( sort keys %{ $value->{$host} } ) {
-                    push @tmp, { k => $k, v => $value->{$host}->{$k} };
-                }
-                push @res, { k => $host, h => \@tmp };
+        $value ||= {};
+        foreach my $host ( sort keys %$value ) {
+            my @tmp;
+            foreach my $k ( sort keys %{ $value->{$host} } ) {
+                push @tmp, { k => $k, v => $value->{$host}->{$k} };
             }
+            push @res, { k => $host, h => \@tmp };
         }
         return $self->sendJSONresponse( $req, { value => \@res } );
     }
@@ -732,7 +873,8 @@ sub getKey {
     # When scalar
     return $self->sendError( $req, "Key $key is not a hash", 400 )
       if ($subkey);
-    return $self->sendError( $req, 'setDefault', 200 ) unless defined($value);
+    return $self->sendError( $req, 'setDefault', 200 )
+      unless defined($value);
     return $self->sendJSONresponse( $req, { value => $value } );
 
     # TODO authParam key
