@@ -6,7 +6,7 @@ use JSON;
 use Lemonldap::NG::Common::PSGI::Constants;
 use Lemonldap::NG::Common::PSGI::Request;
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.8';
 
 our $_json = JSON->new->allow_nonref;
 
@@ -17,6 +17,7 @@ has languages    => ( is => 'rw', isa     => 'Str', default => 'en' );
 has logLevel     => ( is => 'rw', isa     => 'Str', default => 'info' );
 has portal       => ( is => 'rw', isa     => 'Str' );
 has staticPrefix => ( is => 'rw', isa     => 'Str' );
+has instanceName => ( is => 'rw', isa     => 'Str', default => '' );
 has templateDir  => ( is => 'rw', isa     => 'Str|ArrayRef' );
 has links        => ( is => 'rw', isa     => 'ArrayRef' );
 has menuLinks    => ( is => 'rw', isa     => 'ArrayRef' );
@@ -34,21 +35,29 @@ sub init {
     foreach my $k ( keys %$args ) {
         $self->{$k} = $args->{$k} unless ( $k eq 'logger' );
     }
-    unless ( $self->logger and $self->userLogger ) {
+    unless ( ref( $self->logger ) and ref( $self->userLogger ) ) {
         my $logger =
              $args->{logger}
           || $ENV{LLNG_DEFAULTLOGGER}
           || 'Lemonldap::NG::Common::Logger::Std';
-        unless ( $self->logger ) {
+        unless ( ref $self->logger ) {
             eval "require $logger";
             die $@ if ($@);
             $self->logger( $logger->new($self) );
         }
-        unless ( $self->userLogger ) {
+        unless ( ref $self->userLogger ) {
             $logger = $ENV{LLNG_USERLOGGER} || $args->{userLogger} || $logger;
             eval "require $logger";
             die $@ if ($@);
-            $self->userLogger( $logger->new( $self, user => 1 ) );
+            require Lemonldap::NG::Common::Logger::_Duplicate;
+            $self->userLogger(
+                Lemonldap::NG::Common::Logger::_Duplicate->new(
+                    $self,
+                    user   => 1,
+                    logger => $logger,
+                    dup    => $self->logger
+                )
+            );
         }
     }
     return 1;
@@ -112,8 +121,8 @@ sub userError {
 # Responses methods
 sub sendJSONresponse {
     my ( $self, $req, $j, %args ) = @_;
-    $args{code} ||= 200;
-    $args{headers} ||= $req->respHeaders || [];
+    $args{code}    ||= 200;
+    $args{headers} ||= [ $req->spliceHdrs ];
     my $type = 'application/json; charset=utf-8';
     if ( ref $j ) {
         eval { $j = $_json->encode($j); };
@@ -143,9 +152,9 @@ sub sendError {
         return [
             $code,
             [
-                'Content-Type' => 'application/xml; charset=utf-8',
-                @{ $req->respHeaders || [] },
-                'Content-Length' => length($s)
+                'Content-Type'   => 'application/xml; charset=utf-8',
+                'Content-Length' => length($s),
+                $req->spliceHdrs,
             ],
             [$s]
         ];
@@ -180,8 +189,9 @@ body{background:#000;color:#fff;padding:10px 50px;font-family:sans-serif;}a{text
         return [
             $code,
             [
-                'Content-Type' => 'text/html; charset=utf-8',
-                @{ $req->respHeaders || [] }, 'Content-Length' => length($s)
+                'Content-Type'   => 'text/html; charset=utf-8',
+                'Content-Length' => length($s),
+                $req->spliceHdrs,
             ],
             [$s]
         ];
@@ -201,7 +211,7 @@ sub _mustBeDefined {
     my $name = ( caller(1) )[3];
     $name =~ s/^.*:://;
     my $call = ( caller(1) )[0];
-    my $ref = ref( $_[0] ) || $call;
+    my $ref  = ref( $_[0] ) || $call;
     die "$name() method must be implemented (probably in $ref)";
 }
 
@@ -238,8 +248,8 @@ sub sendHtml {
     my $sc = $req->script_name;
     $sc = '.' unless ($sc);
     $sc =~ s#/*$#/#;
-    $args{code} ||= 200;
-    $args{headers} ||= $req->respHeaders || [];
+    $args{code}    ||= 200;
+    $args{headers} ||= [ $req->spliceHdrs ];
     my $htpl;
     $template = ( $args{templateDir} // $self->templateDir ) . "/$template.tpl";
     return $self->sendError( $req, "Unable to read $template", 500 )
@@ -262,6 +272,7 @@ sub sendHtml {
         # TODO: warn if STATICPREFIX does not end with '/'
         $htpl->param(
             STATIC_PREFIX => $sp,
+            INSTANCE_NAME => $self->instanceName,
             SCRIPTNAME    => $sc,
             ( $self->can('tplParams') ? ( $self->tplParams($req) ) : () ),
             (
